/*
 * Make a directory in the filesystem using the client library.
 *
 * Copyright (C) 2019, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/stat.h>
#include <sys/types.h>

#include <e2access/path.h>
#include <fsclient/client.h>

#include "ops.h"
#include "session.h"



/* Alternative metadata set by options. */

extern struct metadata md;



static int _make_dir(const char *path)
{
    char parent[strlen(path) + 1];

    /* Attempt to create the directory and return if successful. */

    long err = client_mkdir(path, 0777 & ~md.mask);

    if (!err)
        return 0;

    /* Without a usable parent directory, attempt to create the parent, and then
       attempt to create the directory again. */

    if (err == -L4_ENOENT)
    {
        strcpy(parent, path);
        path_split(parent);

        if (_make_dir(parent))
            return 1;

        return _make_dir(path);
    }

    fprintf(stderr, "Could not create directory: %s\n", path);
    return 1;
}

static int _make_dirs(const char *path)
{
    struct stat st;

    /* Search for the remaining components. */

    long err = client_stat(path, &st);

    if (!err)
    {
        fprintf(stderr, "Path exists: %s\n", path);
        return 1;
    }

    /* Try and make the directory. */

    return _make_dir(path);
}

/* Make directories in the filesystem. */

int make_dirs(int argc, char *argv[])
{
    int i;

    /* Make each directory component in the given pathname. */

    for (i = 0; i < argc; i++)
        if (_make_dirs(argv[i]))
            return 1;

    return 0;
}

/* vim: tabstop=4 expandtab shiftwidth=4
*/
